// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TicTacToeGame.hpp"
#include "game/DawnGame.hpp"
#include "scene/components/example/ExampleSpin.hpp"
#include "scene/components/physics/3d/CubeCollider.hpp"

using namespace Dawn;

TicTacToeGame::TicTacToeGame(SceneItem *item) : SceneItemComponent(item) {}

void TicTacToeGame::onStart() {
  this->tiles = getScene()->findComponents<TicTacToeTile>();

  getScene()->eventSceneUpdate.addListener(this, &TicTacToeGame::onSceneUpdate);
}

void TicTacToeGame::onSceneUpdate() {
  // Get mouse in screen space.
  auto mouse = getGame()->inputManager.getAxis2D(INPUT_BIND_MOUSE_X, INPUT_BIND_MOUSE_Y);
  mouse *= 2.0f;
  mouse -= glm::vec2(1, 1);

  Camera *camera = getScene()->findComponent<Camera>();
  if(camera == nullptr) return;

  struct Ray3D ray;
  ray.origin = glm::vec3(1,2,3);
  ray.direction = camera->getRayDirectionFromScreenSpace(mouse);

  auto collider = getScene()->findComponent<CubeCollider>();
  if(collider == nullptr) return;

  struct Collider3DRayResult result;
  auto test = collider->raycast(&result, ray);

  if(test) {
    getScene()->debugRay((struct SceneDebugRay){ .start = result.point, .direction = result.normal, .color = COLOR_BLUE });
  } else {
    getScene()->debugRay((struct SceneDebugRay){ .start = ray.origin, .direction = ray.direction });
  }
}